/*******************************************************************************
 * Main object that interacts between the user and the backend.
 ******************************************************************************/
var FAS = {
  
  // nsISupports
  QueryInterface: function FAS_QI(iid) {
    if (iid.equals(Ci.nsIObserver) ||
        iid.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_ERROR_NO_INTERFACE; 
  },
  
  observe: function FAS_observe(aSubject, aTopic, aData) {
  
    // ignore other topics
    if (aTopic != "netscape-fas")
      return;

    // update   
    var self = this;
    window.setTimeout(function() { self.update(); }, 0); 
  },
  
  _obService: null,
  get obService() { return this._obService; },
  set obService(val) { this._obService = val; },
  
  _fasService: null,
  get fasService() { return this._fasService; },
  set fasService(val) { this._fasService = val; },

  onLoad: function FAS_onLoad() {
    
    // Get the observer service and add ourself as an observer
    this.obService = Cc["@mozilla.org/observer-service;1"].
                     getService(Ci.nsIObserverService);
    this.obService.addObserver(this, "netscape-fas", false);
    
    // Get the fas service
    this.fasService = Cc["@netscape.com/fas/service;1"].
                      getService(Ci.nsIFASService);    
      
    // Update the UI
    var self = this;
    window.setTimeout(function() { self.update(); }, 100);                      
  },
  
  onUnload: function FAS_onUnload() {
    
    // remove ourself as an observer
    this.obService.removeObserver(this, "netscape-fas");

    // remove variables
    this.obService = null;
    this.fasService = null;     
  },
  
  getString: function FAS_getString(name, replace) { 
  
    // get the string bundle
    var bundle = document.getElementById("fas_bundle");
    if (!bundle)
      return null;
    
    // return the string  
    if (!replace)
      return bundle.getString(name);
    else
      return bundle.getFormattedString(name, replace);
  },
      
  update: function FAS_update() {

    // Get the FAS Button
    var button = document.getElementById("fas_button");
    if (!button)
      return;
    
    // Update with not signed in
    const FAS_PREFIX = "fas.overlay.tooltip.";    
    if (!this.fasService.isSignedIn) {
      button.removeAttribute("new");
      button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "login"));
      return;
    }
    
    // Update with no activity
    if (!this.fasService.hasActivity) {
      button.removeAttribute("new");
      button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "none"));
      return;
    }
    
    // Update with activity
    var count = String(this.fasService.totalCount);
    button.setAttribute("new", "true");
    if (count == "1")
      button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "single", [count]));
    else
      button.setAttribute("tooltiptext", this.getString(FAS_PREFIX + "plural", [count]));
    
  },
  
  checkActivity: function FAS_checkActivity() {
    var self = this;
    window.setTimeout(function() { self.fasService.checkActivity(); }, 0);
  },
  
  open: function FAS_open(event) {
    if (event.button != 0)
      return;
      
    var el = document.getElementById("fas_panel");
    var sidebar = document.getElementById("sidebar");
    if (el.getAttribute("checked") == "true")
      sidebar.contentWindow.FASPanel.showFAS();
    else
      toggleSidebar("fas_panel", true);
  }
};

window.addEventListener("load", function (event) { FAS.onLoad(); }, false);
window.addEventListener("unload", function(event) { FAS.onUnload(); }, false);